# -*- coding: utf-8 -*-
'''
Created on Sun Dec  8 14:49:19 2024

@author: mstep
'''

# See: https://stackoverflow.com/questions/55619783/tkinter-i-created-a-custom-scrollable-frame-class-is-it-possible-for-it-to-au

import tkinter as tk

class ScrolledFrame(tk.Frame):
    def __init__(self, frame, *args, **kwargs):
        tk.Frame.__init__(self, frame, *args, **kwargs)

        # Make row 0 and column 0 expand.
        self.columnconfigure(0, weight=1)
        self.rowconfigure(0, weight=1)

        # Scrollbars.
        self.v_scroll = tk.Scrollbar(self, orient=tk.VERTICAL)
        self.v_scroll.grid(row=0, column=1, sticky=tk.NSEW)

        self.h_scroll = tk.Scrollbar(self, orient=tk.HORIZONTAL)
        self.h_scroll.grid(row=1, column=0, sticky=tk.NSEW)

        # Canvas.
        self.canvas = tk.Canvas(self,
            xscrollcommand=self.h_scroll.set,
            yscrollcommand=self.v_scroll.set)
        self.canvas.grid(row=0, column=0, sticky=tk.NSEW)

        # Configure the canvas.
        self.v_scroll.config(command=self.canvas.yview)
        self.h_scroll.config(command=self.canvas.xview)

        # Frame.
        self.frame = tk.Frame(self.canvas)
        self.frame_window = self.canvas.create_window(0, 0,
            window=self.frame, anchor=tk.NW)


    def configure_frame(self):
        self.frame.update_idletasks()
        width = self.frame.winfo_reqwidth()
        height = self.frame.winfo_reqheight()
        self.canvas.config(scrollregion=(0, 0, width, height))

        # Bind the mouse wheel to scroll the canvas.
        for child in widget_and_descendants(self.canvas):
            child.bind('<MouseWheel>', self.wheel_scroll)
            #child.bind('<Button-4>', self.wheel_scroll)
            #child.bind('<Button-5>', self.wheel_scroll)

    def wheel_scroll(self, event):
        ''' Scroll the canvas.'''
        # This is the Windows version.
        # Divide event.delta by some number to slow scrolling. Some people
        # recommend dividing by 120 for Windows, but I left it alone to make
        # it scroll about the same speed when the mouse is over the scrollbar.
        units = -int(event.delta / 30)
        self.canvas.yview_scroll(units, 'units')

def widget_and_descendants(parent):
    '''Return this widget and all of its descendants.'''
    results = [parent]
    for child in parent.winfo_children():
        results += widget_and_descendants(child)
    return results

#%%
import tkinter as tk
from PIL import Image, ImageTk
from tkinter.filedialog import askopenfilename, asksaveasfilename

class CropImageApp:
    def __init__(self):
        # Make the main interface.
        self.window = tk.Tk()
        self.window.title('crop_image')
        self.window.protocol('WM_DELETE_WINDOW', self.kill_callback)
        self.window.geometry('500x400')

        # Menus.
        self.build_menus()

        # Make a ScrolledFrame.
        self.scrolled_frame = ScrolledFrame(self.window)
        self.scrolled_frame.pack(side=tk.TOP, fill=tk.BOTH, expand=True)

        # Configure the frame. (Do this after adding all contained widgets.)
        self.scrolled_frame.configure_frame()

        # Currently there is no image or selection rectangle.
        self.image = None
        self.photo_image = None
        self.selection_rectangle = None
        self.selecting = False

        # Create the canvas where we will display images.
        self.canvas = tk.Canvas(self.scrolled_frame.frame, cursor='tcross')
        self.canvas.pack()
        self.scrolled_frame.configure_frame()

        # Watch for mouse events.
        self.canvas.bind('<Button-1>', self.mouse_down)
        self.canvas.bind('<B1-Motion>', self.mouse_move)
        self.canvas.bind('<ButtonRelease-1>', self.mouse_up)

        # Display the window.
        self.window.focus_force()
        self.window.mainloop()

    def build_menus(self):
        '''Build the menu bar.'''
        menu_bar = tk.Menu(self.window)
        self.window.config(menu=menu_bar)

        # Build the File menu.
        self.file_menu = tk.Menu(menu_bar, tearoff=False)
        menu_bar.add_cascade(label='File', menu=self.file_menu)

        # Open
        self.open_photo_image = ImageTk.PhotoImage(file='open.png')        
        self.file_menu.add_command(label='Open...', accelerator='Ctrl+O',
            compound='left',
            command=self.mnu_open, image=self.open_photo_image)
        self.window.bind('<Control-o>', lambda event: self.mnu_open())

        # Save As
        self.save_as_photo_image = ImageTk.PhotoImage(file='save_as.png')        
        self.file_menu.add_command(label='Save As...', accelerator='Ctrl+S',
            compound='left', command=self.mnu_save_as,
            image=self.save_as_photo_image)
        self.file_menu.entryconfig('Save As...', state=tk.DISABLED)
        self.window.bind('<Control-s>', lambda event: self.mnu_save_as())

        # Exit.
        self.file_menu.add_separator()
        self.exit_photo_image = ImageTk.PhotoImage(file='exit.png')        
        self.file_menu.add_command(label='Exit', compound='left',
            command=self.kill_callback, image=self.exit_photo_image)

    def mnu_open(self):
        '''Open a file.'''
        # Get the file name.
        filetypes = [
            ('Image Files', '*.png *.jpg *.bmp *.gif'),
            ('All Files', '*,*')
        ]
        filename = askopenfilename(filetypes=filetypes)
        if len(filename) == 0:
            return

        # Load the selected image.
        self.image = Image.open(filename)
        self.photo_image = ImageTk.PhotoImage(self.image)

        # Remove the selection rectangle.
        self.selection_rectangle = None
        self.file_menu.entryconfig('Save As...', state=tk.DISABLED)

        # Size the canvas to fit the image.
        wid = self.photo_image.width()
        hgt = self.photo_image.height()
        self.canvas.config(width=wid, height=hgt)
        self.scrolled_frame.configure_frame()

        # Display the image.
        self.display_image()

    def display_image(self):
        '''Display the image with a selection rectangle if there is one.'''
        # Delete all canvas contents.
        self.canvas.delete(tk.ALL)

        # Create the new image.
        self.canvas.create_image(1, 1, anchor=tk.NW, image=self.photo_image)

        # Draw the selection rectangle.
        if self.selection_rectangle is not None:
            thickness = 1
            self.canvas.create_rectangle(*self.selection_rectangle,
                width=thickness, outline='red')
            self.canvas.create_rectangle(*self.selection_rectangle,
                width=thickness, outline='yellow',
                dash=(3,5))

    def mnu_save_as(self):
        '''Save the selected part of the image.'''
        # Get the file name.
        filetypes = [
            ('Image Files', '*.png *.jpg *.bmp *.gif'),
            ('All Files', '*,*')
        ]
        filename = asksaveasfilename(
            filetypes=filetypes,
            defaultextension='.png')

        crop_rectangle = self.get_crop_rectangle()
        cropped_image = self.image.crop(crop_rectangle)
        cropped_image.save(filename)

    def get_crop_rectangle(self):
        '''Return the selection rectangle with x1 < x2 and y1 < y2.'''
        x1 = min(self.selection_rectangle[0], self.selection_rectangle[2])
        y1 = min(self.selection_rectangle[1], self.selection_rectangle[3])
        x2 = max(self.selection_rectangle[0], self.selection_rectangle[2])
        y2 = max(self.selection_rectangle[1], self.selection_rectangle[3])
        return [x1, y1, x2, y2]

    def kill_callback(self):
        self.window.destroy()

    def mouse_down(self, event):
        '''Start selecting.'''
        self.selecting = True
        self.selection_rectangle = [event.x, event.y, event.x, event.y]
        self.display_image()

    def mouse_move(self, event):
        '''Update the selection rectangle.'''
        if not self.selecting: return
        self.selection_rectangle[2] = event.x
        self.selection_rectangle[3] = event.y
        self.display_image()

    def mouse_up(self, event):
        '''Stop selecting.'''
        self.selecting = False

        # Enable or disable the Save As menu item.
        disabled = \
            self.photo_image is None or \
            self.selection_rectangle is None or \
            self.selection_rectangle[0] == self.selection_rectangle[2] or \
            self.selection_rectangle[1] == self.selection_rectangle[3]
        if disabled:
            self.file_menu.entryconfig('Save As...', state=tk.DISABLED)
        else:
            self.file_menu.entryconfig('Save As...', state=tk.NORMAL)

#%%

CropImageApp()
print('Done')
